# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of Striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that sends messages to the text player's
statusbar.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class FeedbackManager(object):
	"""
	This class creates an instance that manages messages sent to the
	text player's statusbar.
	"""

	def __init__(self, player):
		"""
		Initialize object.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@param player: Reference to the text player.
		@type player: An player object.
		"""
		self.__init_attributes(player)
		self.__reset_message()

	def __init_attributes(self, player):
		"""
		Initialize data attributes.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@param player: Reference to the text player.
		@type player: An player object.
		"""
		self.__player = player
		self.__icon_dictionary = self.__create_feedback_icons()
		self.__message_stack = []
		self.__message_id = None
		self.__reset_timer = None
		self.__is_busy = False
		self.__default_message_is_set = False
		self.__image = player.glade.get_widget("StatusImage")
		self.__label = player.glade.get_widget("StatusLabel")
		self.__status_id = None
		return

########################################################################
#
#						Public APIs
#
########################################################################

	def update_status_message(self, message=None, icon=None, time=3):
		"""
		Update the statusbar with feedback message and image.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@param message: Message to send to the statusbar.
		@type message: A String object.

		@param icon: Image to place in the statusbar.
		@type icon: A gtk.STOCK object.

		@param time: How long to display feedback message in seconds.
		@type time: An Integer object.

		@return: False to call this function once.
		@rtype: A Boolean object.
		"""
		self.__is_busy = True
		from gobject import idle_add
		idle_add(self.__set_message, message)
		idle_add(self.__set_icon, icon)
		idle_add(self.__reset, time)
		idle_add(self.__beep, icon)
		self.__default_message_is_set = False
		self.__player.response()
		return False
		
	def reset(self):
		self.__reset_message()
		return 

	def set_modal_message(self, message=None, icon=None):
		"""
		Add feedback data to message stack.

		@param message: Feedback data representing a message.
		@type message: A String object.

		@param icon: Feedback data representing an image.
		@type icon: A String object.

		@return: A unique number associated with a set of feedback data.
		@rtype: An Integer object.
		"""
		from utils import generate_random_number
		message_id = generate_random_number(self.__message_stack)
		self.__message_stack.append((message, icon, message_id))
		from gobject import idle_add
		idle_add(self.__set_message, message)
		idle_add(self.__set_icon, icon)
		self.__default_message_is_set = False
		self.__player.response()
		return message_id

	def unset_modal_message(self, message_id, reset=True):
		"""
		Remove feedback data from message stack.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@param message_id: A unique number associated with feedback data.
		@type message_id: A Integer object.

		@param reset: If True reset the statusbar.
		@type reset: A Boolean object.

		@return: False to call this function once.
		@rtype: A Boolean object.
		"""
		from operator import truth
		for feedback in self.__message_stack:
			found = self.__remove_message_from_stack(message_id, feedback)
			if truth(found): break
		if truth(reset):
			from gobject import idle_add
			idle_add(self.__reset_message)
		self.__player.response()
		return False

	def start_spinner(self):
		"""
		Start spinning the feedback throbber.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
#		self.__spinner.start()
		return False

	def stop_spinner(self):
		"""
		Stop spinning the feedback throbber.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
#		self.__spinner.stop()
		return False

	def start_busy_cursor(self):
		"""
		Show the busy cursor.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
		#from cursor import show_busy_textview_cursor
		#show_busy_textview_cursor(self.__player.textview)
		return False

	def stop_busy_cursor(self):
		"""
		Stop showing busy cursor.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
		#from cursor import show_textview_cursor
		#show_textview_cursor(self.__player.textview)
		return False

########################################################################
#
#						Helper Methods
#
########################################################################

	def __set_message(self, message):
		"""
		Update the statusbar with message.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@param message: A message to push to the statusbar.
		@type message: A String object.
		"""
		from operator import not_
		if not_(message): return
		self.__player.response()
		self.__label.set_markup(message)
		self.__player.response()
		return False

	def __set_icon(self, icon):
		"""
		Update the statusbar with icon.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@param message: An icon to place in the statusbar.
		@type message: A gtk.STOCK object.
		"""
		from operator import not_
		if not_(icon): return
		self.__player.response()
		self.__image.clear()
		from gtk import ICON_SIZE_MENU
		self.__image.set_from_stock(self.__icon_dictionary[icon], ICON_SIZE_MENU)
		self.__image.show_all()
		self.__player.response()
		return False

	def __remove_message(self):
		"""
		Remove message from statusbar.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
		self.__player.response()
		self.__label.set_markup("")
		self.__player.response()
		return False

	def __remove_icon(self):
		"""
		Remove image from statusbar.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
		self.__player.response()
		self.__image.clear()
		self.__player.response()
		return False

	def __reset_message(self):
		"""
		Reset the message in the statusbar.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
		from operator import truth
		if truth(self.__message_stack):
			self.__set_message_from_stack()
		else:
			self.__remove_icon()
			self.__remove_message()
			self.__set_icon("no")
			from i18n import msg0002
			self.__set_message(msg0002)
		self.__is_busy = False
		self.__default_message_is_set = True
		return False

	def __set_message_from_stack(self):
		"""
		Determine feedback to show in the statusbar from the message stack.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
		message = self.__message_stack[-1][0]
		icon = self.__message_stack[-1][1]
		self.__set_message(message)
		self.__set_icon(icon)
		return False

	def __remove_message_from_stack(self, message_id, feedback):
		"""
		Remove feedback data from message stack.

		@param message_id: A unique number associated with feedback data.
		@type message_id: An Integer object.

		@param feedback: Data representing feedback information.
		@type feedback: A Tuple object.

		@return: True if feedback data is found in message stack.
		@rtype: A Boolean object.
		"""
		from operator import contains
		if contains(feedback, message_id):
			self.__message_stack.remove(feedback)
			return True
		return False

	def __reset(self, time):
		"""
		Reset the statusbar with default message.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@param time: How long to reset the statusbar.
		@type time: An Integer object.
		"""
		from operator import is_
		if is_(time, None):
			time = 3
		time *= 1000
		from gobject import timeout_add
		self.__remove_timer()
		self.__reset_timer = timeout_add(time, self.__reset_message)
		return False

	def __remove_timer(self):
		"""
		Remove timer of timeout_add functions to prevent multiple calls.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.
		"""
		from gobject import source_remove
		try:
			source_remove(self.__reset_timer)
		except:
			pass
		return False

	def __beep(self, icon):
		"""
		Beep on failed, error or warning messages.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@param icon: A string representing an image.
		@type icon: A String object.
		"""
		from operator import contains
		if contains(("fail", "no", "error", "warning"), icon):
			from gtk.gdk import beep
			beep()
		return False

	def __create_feedback_icons(self):
		"""
		Generate feedback images for the statusbar.

		@param self: Reference to the FeedbackManager instance.
		@type self: A FeedbackManager object.

		@return: A dictionary of stock icons.
		@rtype: A Dictionary object.
		"""
		from gtk import STOCK_YES, STOCK_NO, STOCK_OPEN, STOCK_EDIT, STOCK_SAVE
		from gtk import STOCK_SAVE_AS, STOCK_NEW, STOCK_PRINT, STOCK_UNDO
		from gtk import STOCK_REDO, STOCK_JUMP_TO, STOCK_FIND
		from gtk import STOCK_FIND_AND_REPLACE, STOCK_PREFERENCES
		from gtk import STOCK_DIALOG_WARNING, STOCK_DIALOG_AUTHENTICATION
		from gtk import STOCK_DIALOG_ERROR, STOCK_COPY, STOCK_CUT, STOCK_PASTE
		from gtk import STOCK_SPELL_CHECK, STOCK_HELP, STOCK_EXECUTE
		from gtk import STOCK_DIALOG_INFO, STOCK_ABOUT, STOCK_SELECT_COLOR
		from gtk import STOCK_STOP, STOCK_CONNECT, STOCK_DISCONNECT, STOCK_NETWORK
		from gtk import STOCK_MEDIA_PAUSE, STOCK_MEDIA_PLAY
		icon_dictionary = {
			"succeed": STOCK_YES,
			"suceed": STOCK_YES,
			"fail": STOCK_NO,
			"no": STOCK_NO,
			"yes": STOCK_YES,
			"open": STOCK_OPEN,
			"preferences": STOCK_PREFERENCES,
			"warning": STOCK_DIALOG_WARNING,
			"error": STOCK_DIALOG_ERROR,
			"help": STOCK_HELP,
			"run": STOCK_EXECUTE,
			"info": STOCK_DIALOG_INFO,
			"about": STOCK_ABOUT,
			"stop": STOCK_STOP,
			"play": STOCK_MEDIA_PLAY,
			"pause": STOCK_MEDIA_PAUSE,
			"connect": STOCK_CONNECT,
			"disconnect": STOCK_DISCONNECT,
		}
		return icon_dictionary
